--****************************************************************************
--**
--**  File     :  /lua/unit.lua
--**  Author(s):  John Comes, David Tomandl, Gordon Duclos
--**
--**  Summary  : The Unit lua module
--**
--**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--****************************************************************************

local oldUnit=Unit
Unit = Class(oldUnit) {

    OnCreate = function(self)
        oldUnit.OnCreate(self)
        local bp = self:GetBlueprint()
        if bp.Economy.XPperLevel then -- gh, just to make certain this is not called
            --for units which do not need it
            self.XPnextLevel = bp.Economy.XPperLevel
            self.xp = 0
--            self.Txp = 0 --gh, disabled, ui code works fine
            self.VeteranLevel = 1
            self.LevelProgress = 1
            -- gh, removed BuildRate Sync Entry in order to speed up gameplay, there's an engine method we will use now.
--            self.Sync.XPnextLevel = self.XPnextLevel --Two new sync entries
--            self.Sync.Txp = self.Txp --was slowing things down
            self.Sync.LevelProgress = self.LevelProgress
            self.Sync.RegenRate = bp.Defense.RegenRate
        end
    end,

    OnStopBeingBuilt = function(self, builder, layer)
        if self:GetBlueprint().Economy.xpTimeStep and EntityCategoryContains(categories.ENERGYPRODUCTION + categories.MASSSTORAGE + categories.ENERGYSTORAGE, self) then
            self:ForkThread(self.XPOverTime)
        end--gh, only triggers energy XPOverTime, fixes threading issue
        oldUnit.OnStopBeingBuilt(self, builder, layer)
    end,

    --Economy
    --gh, this is for everything which requires energy to be run: intel, shields etc
    SetMaintenanceConsumptionActive = function(self)
        if self.XPOverTimeThread then KillThread(self.XPOverTimeThread) end
        self.XPOverTimeThread = ForkThread(self.XPOverTime, self)
        oldUnit.SetMaintenanceConsumptionActive(self)
    end,

    XPOverTime = function(self)
        if not self:GetBlueprint().Economy.xpTimeStep then return end -- sanity check
        if self:GetBlueprint().Economy.xpTimeStep <= 0 then return end
        local waittime = self:GetBlueprint().Economy.xpTimeStep * 0.5
        local step = 0.5
        if ALLies != true then step = 0.25 end
        WaitSeconds(waittime) --gh, wait more to improve performance
        while not self:IsDead() and (self.MaintenanceConsumption or EntityCategoryContains(categories.ENERGYPRODUCTION + categories.MASSSTORAGE + categories.ENERGYSTORAGE, self))do
            self:AddXP(self.XPnextLevel* step)
            WaitSeconds(waittime)
        end
    end,

    -- gh, reworked builder xp to include upgrades and allow for pausing
    startBuildXPThread = function(self)
        local levelPerSecond = self:GetBlueprint().Economy.BuildXPLevelpSecond
        if not levelPerSecond then return end
        if  levelPerSecond <= 0 then return end
        if ALLies == false then levelPerSecond = levelPerSecond*0.5 end
        WaitTicks(10)--gh, when you build THAT fast you do not need any xp, reduces autopausing woes
        if not self:IsDead() and self.ActiveConsumption then
            self:AddXP(self.XPnextLevel*levelPerSecond*0.033)
        else
            return
        end
        WaitTicks(10)
        if not self:IsDead() and self.ActiveConsumption then
            self:AddXP(self.XPnextLevel*levelPerSecond*0.033)
        else
            return
        end
        WaitTicks(10)
        while not self:IsDead() and self.ActiveConsumption do
            --gh, check added, now this loop kills itself if there is no consumption
            self:AddXP(self.XPnextLevel*levelPerSecond*0.1)
            WaitTicks(30)
--        while not self:IsDead() and self:IsPaused() then WaitTicks(30) end
--this'd trigger
        end
    end,

    SetActiveConsumptionActive = function(self)
        if self.BuildXPThread then KillThread(self.BuildXPThread) end
        self.BuildXPThread = ForkThread(self.startBuildXPThread, self)
        oldUnit.SetActiveConsumptionActive(self)
    end,
--this applied to builders

--Nukes/Silos
    startSiloXPThread = function(self)
        local levelPerSecond = self:GetBlueprint().Economy.BuildXPLevelpSecond
        if not levelPerSecond then return end
        if levelPerSecond <= 0 then return end
        WaitTicks(10)
        if not self:IsDead() and self:IsUnitState("SiloBuildingAmmo") then
            self:AddXP(self.XPnextLevel*levelPerSecond*0.016)
        else
            return
        end
        WaitTicks(10)
        if not self:IsDead() and self:IsUnitState("SiloBuildingAmmo") then
            self:AddXP(self.XPnextLevel*levelPerSecond*0.016)
        else
            return
        end
        WaitTicks(10)
        if not self:IsDead() and self:IsUnitState("SiloBuildingAmmo") then
            self:AddXP(self.XPnextLevel*levelPerSecond*0.016)
        else
            return
        end
        WaitTicks(10)
        while not self:IsDead() and self:IsUnitState("SiloBuildingAmmo") do
            --gh, added check if silo is building
            self:AddXP(self.XPnextLevel*levelPerSecond*0.05)
            WaitTicks(60)
        end
    end,--gh, seperate thread, we do not want this to interfere on ACUs and SCUs

    OnSiloBuildStart = function(self, weapon)
        if self.SiloXPThread then KillThread(self.SiloXPThread) end
        self.SiloXPThread = ForkThread(self.startSiloXPThread, self)
        oldUnit.OnSiloBuildStart(self, weapon)
    end,

    ------------------------------------------------------------------------------------- VETERANCY
    ---------------------------------------------------------------------------------

    -- use this to go through the AddKills function rather than directly setting veterancy
    SetVeterancy = function(self, veteranLevel)
        veteranLevel = veteranLevel or 0
        if veteranLevel <= 5 then
            return oldUnit.SetVeterancy(self, veteranLevel)
        else
            local bp = self:GetBlueprint()-- change suggested by Gilbot
            if bp.Veteran['Level'..veteranLevel] then
                self:AddKills(bp.Veteran['Level'..veteranLevel])
            else
                WARN('SetVeterancy called on ' .. self:GetUnitId()
                .. ' with veteran level ' .. veteranLevel
                .. ' which was not defined in its BP file. '
                .. ' Veterancy level has not been set.'
                )
            end
        end
    end,

    --Check to see if we should veteran up.
    CheckVeteranLevel = function(self)
        if not self.XPnextLevel then return end
        local bp = self:GetBlueprint()
--        LOG('xp:' .. self.xp.. ' level at:' .. self.XPperLevel)
        while self.xp >= self.XPnextLevel do
            self.xp = self.xp - self.XPnextLevel
            self.XPnextLevel = bp.Economy.XPperLevel * (1+ 0.1*self.VeteranLevel)
            self:SetVeteranLevel(self.VeteranLevel + 1)--gh, fix calling this so often
            --takes too much time
        end
--        if self.xp >= self.XPnextLevel then
--            local count = 0
--            while self.xp >= self.XPnextLevel do
--                self.xp = self.xp - self.XPnextLevel
--                local curLevel = self.VeteranLevel+count
--                self.XPnextLevel = bp.Economy.XPperLevel * (1+  0.1*curLevel)
--                count = count + 1
--            end
--            self:SetVeteranLevel(count+self.VeteranLevel)
--        end --gh, if i ever fix the buff. lua ...
        self.LevelProgress = self.xp / self.XPnextLevel + self.VeteranLevel
        self.Sync.LevelProgress = self.LevelProgress
--        self.Sync.XPnextLevel = self.Txp - self.xp + self.XPnextLevel
        --gh, syncing XP needed for Levelup, calculation within the UI may have minor errors
    end,

--gh, here's the gpg function fixed, lightweight, which is good
--but not very noticeable at low levels
--consider some better solution
    PlayVeteranFx = function(self,time)
        if self.disableSFX == true then return end
        time = time or 1
        CreateAttachedEmitter(self, 0, self:GetArmy(), '/effects/emitters/destruction_explosion_concussion_ring_01_emit.bp'):ScaleEmitter(time)
    end,


    --Set the veteran level to the level specified
    --gh, fixme: Only call this function once on level gain > 1
    SetVeteranLevel = function(self, level)

        --LOG('*DEBUG: '.. self:GetBlueprint().Description .. ' VETERAN UP! LEVEL ', repr(level))
        --local cnt = level - self.VeteranLevel
        self.VeteranLevel = level

--if we loop apply buff from here the buffs don't stack or get created properly.
--too much work to really nail down what is causing this. possibly some issue with
--BuffAffectUnit not updating properly or something else, don't care.

        -- Apply default veterancy buffs
        local buffTypes = {
            'Health',
            'Regen',
            'Vision',
            'Damage',
            'DamageArea',
            'Range',
            'RateOfFire',
            'Speed',
            'BuildRate',
            'Radar',
            'Sonar',
            'OmniRadius',
            'ResourceProduction',
            'Shield'
        }-- gh, can be removed

        --Ghaleon
        --we check if a unit needs a buff, default applied all buffs on any unit levelup
        --Took massive amounts of time to do and possibly inflated ram usage much
        Buff.ApplyBuff( self, 'VeterancyHealth')
        Buff.ApplyBuff( self, 'VeterancyRegen')
        Buff.ApplyBuff( self, 'VeterancyVision')
        local bp = self:GetBlueprint()
        local brain = GetArmyBrain(self:GetArmy())
        local gotweapon = false
        if bp.Weapon then
            for k,v in bp.Weapon do
                if v.Label != 'DeathWeapon' then
--                    LOG(repr(v.Label))
                    Buff.ApplyBuff( self, 'VeterancyDamage')
                    Buff.ApplyBuff( self, 'VeterancyDamageArea')
                    Buff.ApplyBuff( self, 'VeterancyRange')
                    Buff.ApplyBuff( self, 'VeterancyRateOfFire')
                    gotweapon = true
                    break
                end
            end
        end
        if EntityCategoryContains(categories.MOBILE, self) and not EntityCategoryContains(categories.AIR, self) then
            Buff.ApplyBuff( self, 'VeterancySpeed')
        end
        if self:GetBuildRate() and self:GetBuildRate() > 2 then
            Buff.ApplyBuff( self, 'VeterancyBuildRate')
        end
        if bp.Intel.RadarRadius and bp.Intel.RadarRadius > 0 then
            Buff.ApplyBuff( self, 'VeterancyRadar')
        end
        if bp.Intel.SonarRadius and bp.Intel.SonarRadius > 0 then
            Buff.ApplyBuff( self, 'VeterancySonar')
        end
        if bp.Intel.OmniRadius and bp.Intel.OmniRadius > 0 then
            Buff.ApplyBuff( self, 'VeterancyOmniRadius')
        end
        if (bp.Economy.ProductionPerSecondEnergy and bp.Economy.ProductionPerSecondEnergy > 0) or (bp.Economy.ProductionPerSecondMass and bp.Economy.ProductionPerSecondMass > 0) then
            Buff.ApplyBuff( self, 'VeterancyResourceProduction')
        end
        local gotShield = false
        if EntityCategoryContains(categories.COMMAND, self) or EntityCategoryContains(categories.SUBCOMMANDER, self) then

            Buff.ApplyBuff( self, 'VeterancyCommandProduction')

            Buff.ApplyBuff( self, 'VeterancySpeed2')

            if self.VeteranLevel == 101 then
                for i = 1, self:GetWeaponCount() do
                    local wep = self:GetWeapon(i)
                    local wepbp = wep:GetBlueprint()
                    if wep.Label != 'DeathWeapon' then
                        wep:SetFireTargetLayerCaps( 'Air|Land|Water|Seabed|Sub')
--                        wepbp.AboveWaterFireOnly = false
--                        wepbp.AboveWaterTargetsOnly = false
                        wepbp.FireTargetLayerCapsTable = {
                            Land = 'Air|Land|Water|Seabed|Sub',
                            Seabed = 'Air|Land|Water|Seabed|Sub',
                            Water = 'Air|Land|Water|Seabed|Sub',
                        }
--                        wepbp.ManualFire = false
-- --requires updating this line in the engine, too.
--               wep:GetProjectileBlueprint().Physics.DestroyOnWater = false
-- -- doesn't work, would need to use wep:ChangeProjectileBlueprint and prepare a
-- -- modified version for each (S)ACU to not collide with water,
-- -- ... perfect opportunity to change the looks, too.
                    end
                end
            end

            if not EntityCategoryContains(categories.CYBRAN, self) then
                gotShield = true
            end
        end--gh, here we got our ACU and SCU buffs
        if self:GetShield() or gotShield then
            Buff.ApplyBuff( self, 'VeterancyShield')
        end
        -- Get any overriding buffs if they exist
        local bpb = self:GetBlueprint().Buffs
        --Check for unit buffs

        if bpb then
            for bLevel,bData in bpb do
                if (bLevel == 'Any' or bLevel == 'Level'..level) then
                    --gh, kept 'Any' as the Soothsayer *might* need it, do not care enough atm
                    --need to check if i can fully eliminate this one
                    for bType,bValues in bData do
                        local buffName = self:CreateUnitBuff(bLevel,bType,bValues)
                        if buffName then
                            Buff.ApplyBuff( self, buffName )
                        end
                    end
                end
            end
        end

        if gotweapon then -- gh, the Fx for units & revive
            if level == 6 then self.pFx = ForkThread(self.PlayVeteranFx, self,0.5) end
            if math.mod(level,10) == 1 then
                if self.pFx then KillThread(self.pFx) end
                self.pFx = ForkThread(self.PlayVeteranFx,self,level*0.1)
            end
            if math.mod(level,50) == 1 then
                if EntityCategoryContains(categories.MOBILE - categories.AIR, self) then
                    if brain.BrainType == "Human" then
                        local revive = self.Revive or 0
                        self.Revive = revive + 1
                        self.Sync.Revive = self.Revive
                    end
                end
            end
        end


        if brain.BrainType == "Human" and not brain.StorageEnergyTotal then
            brain.StorageEnergyTotal = 0
        end --GiveStorage is bugged, either mass OR energy can be given
        if brain.BrainType == "Human" and not brain.StorageMassTotal then
            brain.StorageMassTotal = 0
        end
        if bp.Economy.StorageMass then
            if brain.BrainType == "Human" then
                brain.StorageMassTotal = brain.StorageMassTotal + (bp.Economy.StorageMass*0.2)
            end
        end
        if bp.Economy.StorageEnergy then
            if brain.BrainType == "Human" then
                brain.StorageEnergyTotal = brain.StorageEnergyTotal + (bp.Economy.StorageEnergy*0.2)
            end
        end
        if brain.ME != 0 then
            if brain.StorageMassTotal != nil then
                brain:GiveStorage("MASS",brain.StorageMassTotal)
            end
        else
            if brain.StorageEnergyTotal then
                brain:GiveStorage("ENERGY",brain.StorageEnergyTotal)
            end
        end
        self:GetAIBrain():OnBrainUnitVeterancyLevel(self, level)
        self:DoUnitCallbacks('OnVeteran')
    end,

    --function to generate the new veterancy buffs
    --gh, do we really need this ?
    CreateUnitBuff = function(self, levelName, buffType, buffValues)

        -- Generate a buff based on the unitId
        local buffName = self:GetUnitId() .. levelName .. buffType
        local buffMinLevel = nil
        local buffMaxLevel = nil
        if buffValues.MinLevel then buffMinLevel = buffValues.MinLevel end
        if buffValues.MaxLevel then buffMaxLevel = buffValues.MaxLevel end


        -- Create the buff if needed
        if not Buffs[buffName] then
            --LOG(buffName .. ': '..buffMinLevel.. ' - '..buffMaxLevel)
            BuffBlueprint {
                MinLevel = buffMinLevel,
                MaxLevel = buffMaxLevel,
                Name = buffName,
                DisplayName = buffName,
                BuffType = buffType,
                Stacks = buffValues.Stacks,
                --self.BuffTypes[buffType].BuffStacks,
                Duration = buffValues.Duration,
                Affects = buffValues.Affects,
            }
        end

        -- Return the buffname so the buff can be applied to the unit
        return buffName
    end,
    --checkthis

    --Allowing buff bonus and adj bonus at same time!!
    UpdateProductionValues = function(self)
        local bpEcon = self:GetBlueprint().Economy
        if not bpEcon then return end
        self:SetProductionPerSecondEnergy( (self.EnergyProdMod or bpEcon.ProductionPerSecondEnergy or 0)* (self.EnergyProdAdjMod or 1) )
        self:SetProductionPerSecondMass( (self.MassProdMod or bpEcon.ProductionPerSecondMass or 0) * (self.MassProdAdjMod or 1) )
    end,

    AddLevels = function(self, levels)
        local bp = self:GetBlueprint()
        local curlevel = self.VeteranLevel
        local percent = self.LevelProgress - curlevel
        local xpAdd = 0
        if levels >= (1-percent) then
            xpAdd = self.XPnextLevel * (1-percent)
            levels=levels-(1-percent)
        else
            xpAdd =self.XPnextLevel * levels
            levels=0
        end
        while levels > 1 do
            levels=levels-1
            curlevel = curlevel +1
            xpAdd=xpAdd + bp.Economy.XPperLevel * (1+ 0.1*curlevel)
        end
        xpAdd=xpAdd + bp.Economy.XPperLevel * (1+ 0.1*(curlevel+1)) * levels
        self:AddXP(xpAdd) --same
    end,

    AddXP = function(self,amount)
        if not self.XPnextLevel then return end
--        self.Txp = self.Txp + (math.ceil(amount))--slowed things down
--        self.Sync.Txp = self.Txp-- Total EXP collected
        self.xp = self.xp + (amount)--removed ceil, hope this speeds things up
--        LOG('___' .. amount .. repr(self:GetBlueprint().Description))
        self:CheckVeteranLevel()
    end,

    OnKilled = function(self, instigator, type, overkillRatio)
        local bp = self:GetBlueprint()
        local brain = GetArmyBrain(self:GetArmy())
        if bp.Economy.StorageEnergy and self.VeteranLevel > 1 then
            if brain.BrainType == "Human" then
                brain.StorageEnergyTotal = brain.StorageEnergyTotal - (bp.Economy.StorageEnergy*0.2)*(self.VeteranLevel - 1)
                brain:GiveStorage("ENERGY",brain.StorageEnergyTotal)
            end
        end
        if bp.Economy.StorageMass and self.VeteranLevel > 1 then
            if brain.BrainType == "Human" then
                brain.StorageMassTotal = brain.StorageMassTotal - (bp.Economy.StorageMass*0.2)*(self.VeteranLevel-1)
                brain:GiveStorage("MASS",brain.StorageMassTotal)
            end
        end
        if self.Revive > 0 then
            local unitEnhancements = import('/lua/enhancementcommon.lua').GetEnhancements(self:GetEntityId())
            local   where  = self:GetPosition()
            local  revivee = CreateUnitHPR(self:GetUnitId(), self:GetArmy(), where[1],where[2],where[3],0,0,0)
            revivee:SetCanBeKilled(false)
            revivee:SetFireState(1)
            if unitEnhancements then
                for k,v in unitEnhancements do
                    if (v == 'StealthGenerator') then
                        revivee:CreateEnhancement('CloakingGenerator')
                    end
                    if (v == 'ShieldHeavy') or (v == 'ShieldGeneratorField') then
                        revivee:CreateEnhancement('Shield')
                    end
                    revivee:CreateEnhancement(v)
                end
            end
            --custom names can not be queried
            local lvl = math.floor(self.VeteranLevel)
            revivee:AddLevels(lvl-10)
            revivee:SetHealth(nil,1)
            local reviveelvl = revivee.VeteranLevel
            local br = revivee:GetBuildRate()
            local brRatio = reviveelvl/br
            revivee:SetBuildRate(0)
            revivee.Revive = self.Revive - 1
            revivee.Sync.Revive = revivee.Revive
            local id = revivee:GetEntityId()
            ForkThread(function()
                local timer = self.VeteranLevel*3
                if timer > 450 then timer = 450 end
                print('Elite Veteran Revived at Level' .. math.floor(revivee.VeteranLevel) ..  ' !!')
                for i = timer,1,-1 do
                    if not revivee:IsDead() then
                        if math.mod(i,10) == 0 then FloatingEntityText(id, i*0.1) end
                        revivee:SetFireState(1)
                        WaitTicks(1)
                    end
                end
                if not revivee:IsDead() then
                    revivee:SetCanBeKilled(true)
                    revivee:SetBuildRate(br)
                    revivee:SetFireState(0)
                end
            end)
        end

        if instigator and IsUnit(instigator) and not instigator:IsDead() then
            if bp.Economy.xpPerHp then
                local sLevel = self.VeteranLevel
                local xpR = sLevel --just to quickly adjust this calculation
                instigator:AddXP(bp.Economy.xpValue*xpR*0.25)
            end
        end
        oldUnit.OnKilled(self, instigator, type, overkillRatio)
    end,

--SetBuildRange = function(self,amount)
----dummy, if arbitrary blueprint stats can be changed
--end,

    CreateShield = function(self, shieldSpec)
        oldUnit.CreateShield(self, shieldSpec)
        Buff.BuffAffectUnit(self,'VeterancyShield', self, true)
    end,

    CreatePersonalShield = function(self, shieldSpec)
        oldUnit.CreatePersonalShield(self, shieldSpec)
        Buff.BuffAffectUnit(self,'VeterancyShield', self, true)
    end,

    InitiateTeleportThread = function(self, teleporter, location, orientation)
        local tbp = teleporter:GetBlueprint()
        local ubp = self:GetBlueprint()
        self.UnitBeingTeleported = self
        self:SetImmobile(true)
        self:PlayUnitSound('TeleportStart')
        self:PlayUnitAmbientSound('TeleportLoop')
        local bp = self:GetBlueprint().Economy
        local energyCost, time
        if bp then
            local mass = bp.BuildCostMass * (bp.TeleportMassMod or 0.01)
            local energy = bp.BuildCostEnergy * (bp.TeleportEnergyMod or 0.01)
            energyCost = mass + energy
            time = energyCost * (bp.TeleportTimeMod or 0.01)
            if ALLies == false then
                --MP balance
                if EntityCategoryContains(categories.COMMAND, self) then
                    if (time - self.VeteranLevel*0.2) > 3 then --16 tops 3 min at level 80
                        time = time - (self.VeteranLevel*0.2)
                    else
                        time = 3
                    end
                elseif EntityCategoryContains(categories.SUBCOMMANDER, self) then
                    if (time - self.VeteranLevel*0.4) > 6 then --30-38 tops ,6 min
                        time = time - (self.VeteranLevel*0.4)
                    else
                        time = 6
                    end
                else
                    if (time - self.VeteranLevel) > 10 then --30-38 tops ,6 min
                        time = time - (self.VeteranLevel)
                    else
                        time = 10
                    end

                end
            else--SOLO,Survival
                if EntityCategoryContains(categories.COMMAND, self) then
                    if (time - self.VeteranLevel*0.5) > 0 then --16 tops 0,1 min at level 30
                        time = time - (self.VeteranLevel*0.5)
                    else
                        time = 0.1
                    end

                elseif EntityCategoryContains(categories.SUBCOMMANDER, self) then
                    if (time - self.VeteranLevel) > 0.2 then --30-38 tops ,6 min
                        time = time - (self.VeteranLevel)
                    else
                        time = 0.2
                    end

                else
                    if (time - self.VeteranLevel) > 1 then --16 tops 0,1 min at level 30
                        time = time - (self.VeteranLevel)
                    else
                        time = 1
                    end
                end
            end
            -- gh, veterans teleport faster
            -- too bad this can't be done with less code
        end

        --LOG('*UNIT DEBUG: TELEPORTINgh, energyCost = ', repr(energyCost), ' time = ', repr(time))
        self.TeleportDrain = CreateEconomyEvent(self, energyCost or 100, 0, time or 5, self.UpdateTeleportProgress)

        -- create teleport charge effect
        self:PlayTeleportChargeEffects()

        WaitFor( self.TeleportDrain  ) -- Perform fancy Teleportation FX here

        if self.TeleportDrain then
            RemoveEconomyEvent(self, self.TeleportDrain )
            self.TeleportDrain = nil
        end

        self:PlayTeleportOutEffects()
        self:CleanupTeleportChargeEffects()

        WaitSeconds( 0.1 )

        self:SetWorkProgress(0.0)
        Warp(self, location, orientation)
        self:PlayTeleportInEffects()

        WaitSeconds( 0.1 ) -- Perform cooldown Teleportation FX here
        --Landing Sound
        --LOG('DROP')
        self:StopUnitAmbientSound('TeleportLoop')
        self:PlayUnitSound('TeleportEnd')
        self:SetImmobile(false)
        self.UnitBeingTeleported = nil
        self.TeleportThread = nil
    end,

    GetShield = function(self)
        return self.MyShield or nil
    end,
}
