--****************************************************************************
--**  Summary  :  Seraphim Commander Script
--****************************************************************************

local oldXSL0001 = XSL0001
XSL0001 = Class( oldXSL0001 ) {

    RegenBuffThread = function(self)
        while not self:IsDead() do
            --Get friendly units in the area (including self)
            local units = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.ALLUNITS, self:GetPosition(), self:GetBlueprint().Enhancements.RegenAura.Radius)

            --Give them a 5 second regen buff
            for _,unit in units do
                Buff.ApplyBuff(unit, 'SeraphimACURegenAura')
            end

            --Wait 5 seconds
            WaitSeconds(5)
        end
    end,

    AdvancedRegenBuffThread = function(self)
        while not self:IsDead() do
            --Get friendly units in the area (including self)
            local units = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.ALLUNITS, self:GetPosition(), self:GetBlueprint().Enhancements.AdvancedRegenAura.Radius)

            --Give them a 5 second regen buff
            for _,unit in units do
                Buff.ApplyBuff(unit, 'SeraphimAdvancedACURegenAura')
            end

            --Wait 5 seconds
            WaitSeconds(5)
        end
    end,

    OnStartBuild = function(self, unitBeingBuilt, order)
        --# FA version didn't call its own base class
        --# So we'll call it for them.
        SWalkingLandUnit.OnStartBuild(self, unitBeingBuilt, order)
        oldXSL0001.OnStartBuild(self, unitBeingBuilt, order)
    end,

    CreateEnhancement = function(self, enh)
        SWalkingLandUnit.CreateEnhancement(self, enh)
        local bp = self:GetBlueprint().Enhancements[enh]

        -- Regenerative Aura
        if enh == 'RegenAura' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not Buffs['SeraphimACURegenAura'] then
                BuffBlueprint {
                    Name = 'SeraphimACURegenAura',
                    DisplayName = 'SeraphimACURegenAura',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'ALWAYS',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            --Add = bp.RegenPerSecond or 0.1,
                            --Mult = 0,
                            Add = 0,
                            Mult = bp.RegenPerSecond or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }

            end

            table.insert( self.ShieldEffectsBag, CreateAttachedEmitter( self, 'XSL0001', self:GetArmy(), '/effects/emitters/seraphim_regenerative_aura_01_emit.bp' ) )
            self.RegenThreadHandle = self:ForkThread(self.RegenBuffThread)

        elseif enh == 'RegenAuraRemove' then
            if self.ShieldEffectsBag then
                for k, v in self.ShieldEffectsBag do
                    v:Destroy()
                end
                self.ShieldEffectsBag = {}
            end
            KillThread(self.RegenThreadHandle)

        elseif enh == 'AdvancedRegenAura' then
            if self.RegenThreadHandle then
                if self.ShieldEffectsBag then
                    for k, v in self.ShieldEffectsBag do
                        v:Destroy()
                    end
                    self.ShieldEffectsBag = {}
                end
                KillThread(self.RegenThreadHandle)

            end

            local bp = self:GetBlueprint().Enhancements[enh]
            if not Buffs['SeraphimAdvancedACURegenAura'] then
                BuffBlueprint {
                    Name = 'SeraphimAdvancedACURegenAura',
                    DisplayName = 'SeraphimAdvancedACURegenAura',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'ALWAYS',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            --Add = bp.RegenPerSecond or 0.1,
                            --Mult = 0,
                            Add = 0,
                            Mult = bp.RegenPerSecond or 0.1,

                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                        MaxHealth = {
                            Add = 0,
                            Mult = bp.MaxHealthFactor or 1.0,
                        },
                    },
                }
            end

            table.insert( self.ShieldEffectsBag, CreateAttachedEmitter( self, 'XSL0001', self:GetArmy(), '/effects/emitters/seraphim_regenerative_aura_01_emit.bp' ) )
            self.AdvancedRegenThreadHandle = self:ForkThread(self.AdvancedRegenBuffThread)

        elseif enh == 'AdvancedRegenAuraRemove' then
            if self.ShieldEffectsBag then
                for k, v in self.ShieldEffectsBag do
                    v:Destroy()
                end
                self.ShieldEffectsBag = {}
            end
            KillThread(self.AdvancedRegenThreadHandle)

            --Resource Allocation
        elseif enh == 'ResourceAllocation' then
            if not Buffs['SeraACUResourceAllocation'] then
                BuffBlueprint {
                    Name = 'SeraACUResourceAllocation',
                    DisplayName = 'SeraACUResourceAllocation',
                    BuffType = 'SeraACUResourceAllocation',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        MassProductionBuf = {
                            Add =  bp.ProductionPerSecondMass,
                            Mult = 1,
                        },
                        EnergyProductionBuf = {
                            Add = bp.ProductionPerSecondEnergy,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SeraACUResourceAllocation')
        elseif enh == 'ResourceAllocationRemove' then
            Buff.RemoveBuff(self,'SeraACUResourceAllocation',false)
        elseif enh == 'ResourceAllocationAdvanced' then
            if not Buffs['SeraACUResourceAllocationAdvanced'] then
                BuffBlueprint {
                    Name = 'SeraACUResourceAllocationAdvanced',
                    DisplayName = 'SeraACUResourceAllocationAdvanced',
                    BuffType = 'SeraACUResourceAllocationAdvanced',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        MassProductionBuf = {
                            Add =  bp.ProductionPerSecondMass,
                            Mult = 1,
                        },
                        EnergyProductionBuf = {
                            Add = bp.ProductionPerSecondEnergy,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SeraACUResourceAllocationAdvanced')
            Buff.RemoveBuff(self,'SeraACUResourceAllocation',false)
        elseif enh == 'ResourceAllocationAdvancedRemove' then
            Buff.RemoveBuff(self,'SeraACUResourceAllocationAdvanced',false)
            --Damage Stabilization
            --T2 Engineering
        elseif enh =='AdvancedEngineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SeraphimACUT2BuildRate'] then
                BuffBlueprint {
                    Name = 'SeraphimACUT2BuildRate',
                    DisplayName = 'SeraphimACUT2BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate,
                            Mult = 1,
                        },
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SeraphimACUT2BuildRate')
            --T3 Engineering
        elseif enh =='T3Engineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SeraphimACUT3BuildRate'] then
                BuffBlueprint {
                    Name = 'SeraphimACUT3BuildRate',
                    DisplayName = 'SeraphimCUT3BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate,
                            Mult = 1,
                        },
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SeraphimACUT3BuildRate')
            if Buff.HasBuff( self, 'SeraphimACUT2BuildRate' ) then
                Buff.RemoveBuff( self, 'SeraphimACUT2BuildRate' )
            end
            --Blast Attack
        elseif enh == 'BlastAttack' then
            if not Buffs['SeraACUBlastAttack'] then
                BuffBlueprint {
                    Name = 'SeraACUBlastAttack',
                    DisplayName = 'SeraACUBlastAttack',
                    BuffType = 'SeraACUBlastAttack',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        Damage = {
                            Add =  bp.AdditionalDamage,
                            Mult = 1,
                            ByName = {
                                ChronotronCannon = true,
                            },
                        },
                        MaxRadius = {
                            Add =  bp.NewMaxRadius,
                            Mult = 1,
                            ByName = {
                                ChronotronCannon = true,
                            },
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SeraACUBlastAttack')
        elseif enh == 'BlastAttackRemove' then
            if Buff.HasBuff( self, 'SeraACUBlastAttack' ) then
                Buff.RemoveBuff( self, 'SeraACUBlastAttack' )
            end
            --Heat Sink Augmentation
        elseif enh == 'RateOfFire' then
            if not Buffs['SeraACURateOfFire'] then
                BuffBlueprint {
                    Name = 'SeraACURateOfFire',
                    DisplayName = 'SeraACURateOfFire',
                    BuffType = 'SeraACURateOfFire',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        RateOfFireBuf = {
                            Add =  bp.NewRateOfFire,
                            Mult = 1,
                            ByName = {
                                ChronotronCannon = true,
                            },
                        },
                        MaxRadius = {
                            Add =  bp.NewMaxRadius,
                            Mult = 1,
                            ByName = {
                                ChronotronCannon = true,
                                OverCharge = true,
                            },
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SeraACURateOfFire')
        elseif enh == 'RateOfFireRemove' then
            if Buff.HasBuff( self, 'SeraACURateOfFire' ) then
                Buff.RemoveBuff( self, 'SeraACURateOfFire' )
            end
        else
            oldXSL0001.CreateEnhancement(self, enh)
        end
    end,

}

TypeClass = XSL0001